# Benchmark Format

The format of the benchmarks is as follows:
- 1 line with number of servers (N)
- N lines with server descriptions: server id, CPU capacity, memory capacity, idle energy consumption and maximum energy consumption
- 1 line with number of virtual machines (M)
- M lines with virtual machine descriptions: job id, virtual machine index in job, CPU requirements, memory requirements, Boolean that is True if virtual machine cannot be co-located with other virtual machines in the same job (False otherwise); some virtual machines in some instances may include a comma separated list of server ids corresponding to servers where that virtual machine is not allowed to run, referred to as incompatible servers; these can be ignored using the **-ip** option when running vmalloc.jar
- 1 line with number of pre-existing mappings (K)
- K lines with mapping descriptions: job id, virtual machine index in job, server id

File names indicate the type of benchmark.
For example, an instance with the name s32-vm25p-map50p-1.desc indicates that the instance is composed of 32 servers, with a sum of virtual machine requirements comprising more or less 25% of the total server capacity.
*map50p* indicates that the sum of the requirements of the virtual machines in the pre-existing mappings comprise more or less 50% of the total virtual machine requirements.

More precisely, an instance has the following structure:

```
<number of servers (N)>
<server id 1> <cpu capacity 1> <memory capacity 1> <idle energy consumption 1> <maximum energy consumption 1>
    ...
<server id N> <cpu capacity N> <memory capacity N> <idle energy consumption N> <maximum energy consumption N>
<number of virtual machines (M)>
<job id 1> <virtual machine index 1> <cpu requirements 1> <memory requirements 1> <is anti co-locatable 1>
    ...
<job id M> <virtual machine index M> <cpu requirements M> <memory requirements M> <is anti co-locatable M>
<number of mappings (K)>
<mapping job id 1> <mapping virtual machine index 1> <mapping server id 1>
    ...
<mapping job id K> <mapping virtual machine index K> <mapping server id K>
```

# Instructions - vmalloc.jar

A jar is provided that allows one to run the algorithms evaluated in our SAT17 submission.
The source needs a little clean-up work but will be made available eventually when it is ready.

## Running vmalloc.jar

**Always** provide a time limit using the **-t** option, some algorithms (like the evolutionary ones) do not work properly without one.
The **-m** option is used to specify a migration budget percentile. For example, if a percentile of 0.05 is provided, then the sum of the memory requirements of the virtual machines in the pre-existing mappings migrated to some other server cannot exceed the total memory capacity times 0.05.
The **-dp** option specifies a file in which to store the solution set produced by the algorithm, in the format used by the MOEA Framework (version 2.9.1) (http://moeaframework.org/).
For randomized algorithms, **-ms** is used to specify how many seeds to use. In that case, the time limit provided using **-t** is used for each seed.

### PCLD

```
java -server -jar vmalloc.jar -ip -a PCLD -ht NONE -t <time limit> -m <migration budget percentile> -dp <solution set file path> <instance file path>
```

### GIA

```
java -server -jar vmalloc.jar -ip -a GIA -ht NONE -t <time limit> -m <migration budget percentile> -dp <solution set file path> <instance file path>
```

### VMPMBBO

```
java -server -jar vmalloc.jar -ip -a BBO -it SFF -t <time limit> -m <migration budget percentile> -dp <solution set file path> <instance file path>
```

### NSGAII

```
java -server -jar vmalloc.jar -ip -a GA -it SFF -ps 100 -cr 0.8 -mr 0.05 -t <time limit> -m <migration budget percentile> -dp <solution set file path> <instance file path>
```

### MGGA

```
java -server -jar vmalloc.jar -ip -a GGA -it SFF -ps 12 -cr 0.8 -mr 0.0 -t <time limit> -m <migration budget percentile> -dp <solution set file path> <instance file path>
```

## Hypervolume Computation

vmalloc.jar also does hypervolume computation, given the solution set file produced using the **-dp** option and the instance file, as follows:

```
java -jar vmalloc.jar -ip -a <algorithm> -ap DUMMY-LABEL:<solution set file path> <instance file path>
```

